# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: compas_pb/generated/mesh.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'compas_pb/generated/mesh.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from compas_pb.generated import point_pb2 as compas__pb_dot_generated_dot_point__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x63ompas_pb/generated/mesh.proto\x12\x0e\x63ompas_pb.data\x1a\x1f\x63ompas_pb/generated/point.proto\"\x1b\n\x08\x46\x61\x63\x65List\x12\x0f\n\x07indices\x18\x01 \x03(\r\"\x98\x01\n\x08MeshData\x12\x11\n\x04guid\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04name\x18\x02 \x01(\tH\x01\x88\x01\x01\x12+\n\x08vertices\x18\x03 \x03(\x0b\x32\x19.compas_pb.data.PointData\x12\'\n\x05\x66\x61\x63\x65s\x18\x04 \x03(\x0b\x32\x18.compas_pb.data.FaceListB\x07\n\x05_guidB\x07\n\x05_nameb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compas_pb.generated.mesh_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FACELIST']._serialized_start=83
  _globals['_FACELIST']._serialized_end=110
  _globals['_MESHDATA']._serialized_start=113
  _globals['_MESHDATA']._serialized_end=265
# @@protoc_insertion_point(module_scope)
