# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: compas_pb/generated/circle.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'compas_pb/generated/circle.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from compas_pb.generated import frame_pb2 as compas__pb_dot_generated_dot_frame__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n compas_pb/generated/circle.proto\x12\x0e\x63ompas_pb.data\x1a\x1f\x63ompas_pb/generated/frame.proto\"b\n\nCircleData\x12\x0c\n\x04guid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0e\n\x06radius\x18\x03 \x01(\x02\x12(\n\x05\x66rame\x18\x04 \x01(\x0b\x32\x19.compas_pb.data.FrameDatab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compas_pb.generated.circle_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CIRCLEDATA']._serialized_start=85
  _globals['_CIRCLEDATA']._serialized_end=183
# @@protoc_insertion_point(module_scope)
